import datetime
from typing import Dict, List, Union, Literal, Optional

from .model import *
from .types import *

class ApiClient:
    async def get_application_role_connection_metadata_records(
        self, *, application_id: SnowflakeType
    ) -> List[ApplicationRoleConnectionMetadata]:
        """get application role connection metadata records

        see https://discord.com/developers/docs/resources/application-role-connection-metadata#get-application-role-connection-metadata-records
        """
        ...
    async def update_application_role_connection_metadata_records(
        self, *, application_id: SnowflakeType
    ) -> List[ApplicationRoleConnectionMetadata]:
        """get application role connection metadata records

        see https://discord.com/developers/docs/resources/application-role-connection-metadata#get-application-role-connection-metadata-records
        """
        ...
    async def get_guild_audit_log(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: Optional[SnowflakeType] = ...,
        action_type: Optional[AuditLogEventType] = ...,
        before: Optional[SnowflakeType] = ...,
        after: Optional[SnowflakeType] = ...,
        limit: Optional[int] = ...,
    ) -> AuditLog:
        """get guild audit log

        see https://discord.com/developers/docs/resources/audit-log#get-guild-audit-log
        """
        ...
    async def list_auto_moderation_rules_for_guild(
        self, *, guild_id: SnowflakeType
    ) -> List[AutoModerationRule]:
        """list auto moderation rules for guild

        see https://discord.com/developers/docs/resources/auto-moderation#list-auto-moderation-rules-for-guild
        """
        ...
    async def get_auto_moderation_rule(
        self, *, guild_id: SnowflakeType, rule_id: SnowflakeType
    ) -> AutoModerationRule:
        """get auto moderation rule

        see https://discord.com/developers/docs/resources/auto-moderation#get-auto-moderation-rule
        """
        ...
    async def create_auto_moderation_rule(
        self, *, guild_id: SnowflakeType, **data
    ) -> AutoModerationRule:
        """create auto moderation rule

        see https://discord.com/developers/docs/resources/auto-moderation#create-auto-moderation-rule
        """
        ...
    async def modify_auto_moderation_rule(
        self,
        *,
        guild_id: SnowflakeType,
        rule_id: SnowflakeType,
        name: str,
        event_type: AutoModerationRuleEventType,
        trigger_type: TriggerType,
        trigger_metadata: Optional[TriggerMetadata] = ...,
        actions: List[AutoModerationAction] = ...,
        enabled: Optional[bool] = ...,
        exempt_roles: List[SnowflakeType] = ...,
        exempt_channels: List[SnowflakeType] = ...,
        reason: Optional[str] = ...,
    ) -> AutoModerationRule:
        """modify auto moderation rule

        see https://discord.com/developers/docs/resources/auto-moderation#modify-auto-moderation-rule
        """
        ...
    async def delete_auto_moderation_rule(
        self,
        *,
        guild_id: SnowflakeType,
        rule_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """delete auto moderation rule

        see https://discord.com/developers/docs/resources/auto-moderation#delete-auto-moderation-rule
        """
        ...
    async def get_channel(self, *, channel_id: SnowflakeType) -> Channel:
        """get channel

        see https://discord.com/developers/docs/resources/channel#get-channel"""
        ...
    async def modify_DM(
        self,
        *,
        channel_id: SnowflakeType,
        name: str = ...,
        icon: bytes = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """modify DM

        see https://discord.com/developers/docs/resources/channel#modify-channel-json-params-group-dm
        """
        ...
    async def modify_channel(
        self,
        *,
        channel_id: SnowflakeType,
        name: Optional[str] = ...,
        type: Optional[ChannelType] = ...,
        position: Optional[int] = ...,
        topic: Optional[str] = ...,
        nsfw: Optional[bool] = ...,
        rate_limit_per_user: Optional[int] = ...,
        bitrate: Optional[int] = ...,
        user_limit: Optional[int] = ...,
        permission_overwrites: Optional[List[Overwrite]] = ...,
        parent_id: Optional[SnowflakeType] = ...,
        rtc_region: Optional[str] = ...,
        video_quality_mode: Optional[VideoQualityMode] = ...,
        default_auto_archive_duration: Optional[int] = ...,
        flags: Optional[ChannelFlags] = ...,
        available_tags: Optional[List[ForumTag]] = ...,
        default_reaction_emoji: Optional[DefaultReaction] = ...,
        default_thread_rate_limit_per_user: Optional[int] = ...,
        default_sort_order: Optional[SortOrderTypes] = ...,
        default_forum_layout: Optional[ForumLayoutTypes] = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """modify channel

        see https://discord.com/developers/docs/resources/channel#modify-channel-json-params-guild-channel
        """
        ...
    async def modify_thread(
        self,
        *,
        channel_id: SnowflakeType,
        name: str = ...,
        archived: bool = ...,
        auto_archive_duration: int = ...,
        locked: bool = ...,
        invitable: bool = ...,
        rate_limit_per_user: Optional[int] = ...,
        flags: Optional[ChannelFlags] = ...,
        applied_tags: Optional[List[SnowflakeType]] = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """modify thread

        see https://discord.com/developers/docs/resources/channel#modify-channel-json-params-thread
        """
        ...
    async def delete_channel(
        self, *, channel_id: SnowflakeType, reason: Optional[str] = ...
    ) -> Channel:
        """delete channel

        see https://discord.com/developers/docs/resources/channel#deleteclose-channel"""
        ...
    async def get_channel_messages(
        self,
        *,
        channel_id: SnowflakeType,
        around: Optional[SnowflakeType] = ...,
        before: Optional[SnowflakeType] = ...,
        after: Optional[SnowflakeType] = ...,
        limit: Optional[int] = ...,
    ) -> List[MessageGet]:
        """get channel messages

        see https://discord.com/developers/docs/resources/channel#get-channel-messages
        """
        ...
    async def get_channel_message(
        self, *, channel_id: SnowflakeType, message_id: SnowflakeType
    ) -> MessageGet:
        """get channel message

        see https://discord.com/developers/docs/resources/channel#get-channel-message"""
        ...
    async def create_message(
        self,
        *,
        channel_id: SnowflakeType,
        content: Optional[str] = ...,
        nonce: Optional[Union[int, str]] = ...,
        tts: Optional[bool] = ...,
        embeds: Optional[List[Embed]] = ...,
        allowed_mentions: Optional[AllowedMention] = ...,
        message_reference: Optional[MessageReference] = ...,
        components: Optional[List[DirectComponent]] = ...,
        sticker_ids: Optional[List[SnowflakeType]] = ...,
        files: Optional[List[File]] = ...,
        attachments: Optional[List[AttachmentSend]] = ...,
        flags: Optional[MessageFlag] = ...,
    ) -> MessageGet:
        """create message

        see https://discord.com/developers/docs/resources/channel#create-message
        """
        ...
    async def crosspost_message(
        self, *, channel_id: SnowflakeType, message_id: SnowflakeType
    ) -> MessageGet:
        """crosspost message

        see https://discord.com/developers/docs/resources/channel#crosspost-message"""
        ...
    async def create_reaction(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        emoji: str,
        emoji_id: Optional[SnowflakeType] = None,
    ) -> None:
        """create reaction

        see https://discord.com/developers/docs/resources/channel#create-reaction"""
        ...
    async def delete_own_reaction(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        emoji: str,
        emoji_id: Optional[SnowflakeType] = None,
    ) -> None:
        """delete own reaction

        see https://discord.com/developers/docs/resources/channel#delete-own-reaction"""
        ...
    async def delete_user_reaction(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        user_id: SnowflakeType,
        emoji: str,
        emoji_id: Optional[SnowflakeType] = None,
    ) -> None:
        """delete user reaction

        see https://discord.com/developers/docs/resources/channel#delete-user-reaction
        """
        ...
    async def get_reactions(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        emoji: str,
        emoji_id: Optional[SnowflakeType] = None,
        after: Optional[SnowflakeType] = ...,
        limit: Optional[int] = ...,
    ) -> List[User]:
        """get reactions

        see https://discord.com/developers/docs/resources/channel#get-reactions"""
        ...
    async def delete_all_reactions(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
    ) -> None:
        """

        see https://discord.com/developers/docs/resources/channel#delete-all-reactions
        """
        ...
    async def delete_all_reactions_for_emoji(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        emoji: str,
        emoji_id: Optional[SnowflakeType] = None,
    ) -> None:
        """

        see https://discord.com/developers/docs/resources/channel#delete-all-reactions
        """
        ...
    async def edit_message(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        content: Optional[str] = ...,
        embeds: Optional[List[Embed]] = ...,
        flags: Optional[MessageFlag] = ...,
        allowed_mentions: Optional[AllowedMention] = ...,
        components: Optional[List[DirectComponent]] = ...,
        files: Optional[List[File]] = ...,
        attachments: Optional[List[AttachmentSend]] = ...,
    ) -> MessageGet:
        """see https://discord.com/developers/docs/resources/channel#edit-message"""
        ...
    async def delete_message(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#delete-message"""
        ...
    async def bulk_delete_message(
        self,
        *,
        channel_id: SnowflakeType,
        messages: List[SnowflakeType],
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#bulk-delete-messages"""
        ...
    async def edit_channel_permissions(
        self,
        *,
        channel_id: SnowflakeType,
        overwrite_id: SnowflakeType,
        allow: Optional[str] = ...,
        deny: Optional[str] = ...,
        type: Optional[OverwriteType] = ...,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#edit-channel-permissions"""
        ...
    async def get_channel_invites(self, *, channel_id: SnowflakeType) -> List[Invite]:
        """https://discord.com/developers/docs/resources/channel#get-channel-invites"""
        ...
    async def create_channel_invite(
        self,
        *,
        channel_id: SnowflakeType,
        max_age: Optional[int] = ...,
        max_uses: Optional[int] = ...,
        temporary: Optional[bool] = ...,
        unique: Optional[bool] = ...,
        target_type: Optional[int] = ...,
        target_user_id: Optional[SnowflakeType] = ...,
        target_application_id: Optional[SnowflakeType] = ...,
        reason: Optional[str] = ...,
    ) -> Invite:
        """https://discord.com/developers/docs/resources/channel#create-channel-invite"""
        ...
    async def delete_channel_permission(
        self,
        *,
        channel_id: SnowflakeType,
        overwrite_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#delete-channel-permission"""
        ...
    async def follow_announcement_channel(
        self, *, channel_id: SnowflakeType, webhook_channel_id: SnowflakeType = ...
    ) -> FollowedChannel:
        """https://discord.com/developers/docs/resources/channel#follow-news-channel"""
        ...
    async def trigger_typing_indicator(self, *, channel_id: SnowflakeType) -> None:
        """https://discord.com/developers/docs/resources/channel#trigger-typing-indicator"""
        ...
    async def get_pinned_messages(self, *, channel_id: SnowflakeType) -> List[MessageGet]:
        """https://discord.com/developers/docs/resources/channel#get-pinned-messages"""
        ...
    async def pin_message(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#add-pinned-channel-message"""
        ...
    async def unpin_message(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#delete-pinned-channel-message"""
        ...
    async def group_DM_add_recipient(
        self,
        *,
        channel_id: SnowflakeType,
        user_id: SnowflakeType,
        access_token: str = ...,
        nick: str = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#group-dm-add-recipient"""
        ...
    async def group_DM_remove_recipient(
        self, *, channel_id: SnowflakeType, user_id: SnowflakeType
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#group-dm-remove-recipient"""
        ...
    async def start_thread_from_message(
        self,
        *,
        channel_id: SnowflakeType,
        message_id: SnowflakeType,
        name: str = ...,
        auto_archive_duration: Optional[int] = ...,
        rate_limit_per_user: Optional[int] = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """https://discord.com/developers/docs/resources/channel#start-thread-with-message"""
        ...
    async def start_thread_without_message(
        self,
        *,
        channel_id: SnowflakeType,
        name: str = ...,
        auto_archive_duration: Optional[int] = ...,
        type: Optional[ChannelType] = ...,
        invitable: Optional[bool] = ...,
        rate_limit_per_user: Optional[int] = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """https://discord.com/developers/docs/resources/channel#start-thread-without-message"""
        ...

        # TODO: Returns a channel, with a nested message object, on success
    async def start_thread_in_forum_channel(
        self,
        *,
        channel_id: SnowflakeType,
        name: str = ...,
        auto_archive_duration: Optional[int] = ...,
        rate_limit_per_user: Optional[int] = ...,
        applied_tags: Optional[List[SnowflakeType]] = ...,
        content: Optional[str] = ...,
        embeds: Optional[List[Embed]] = ...,
        allowed_mentions: Optional[AllowedMention] = ...,
        components: Optional[List[DirectComponent]] = ...,
        sticker_ids: Optional[List[SnowflakeType]] = ...,
        files: Optional[List[File]] = ...,
        attachments: Optional[List[AttachmentSend]] = ...,
        flags: Optional[MessageFlag] = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """https://discord.com/developers/docs/resources/channel#start-thread-in-forum-channel"""
        ...
    async def join_thread(self, *, channel_id: SnowflakeType) -> None:
        """https://discord.com/developers/docs/resources/channel#join-thread"""
        ...
    async def add_thread_member(
        self, *, channel_id: SnowflakeType, user_id: SnowflakeType
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#add-thread-member"""
        ...
    async def leave_thread(self, *, channel_id: SnowflakeType) -> None:
        """https://discord.com/developers/docs/resources/channel#leave-thread"""
        ...
    async def remove_thread_member(
        self, *, channel_id: SnowflakeType, user_id: SnowflakeType
    ) -> None:
        """https://discord.com/developers/docs/resources/channel#remove-thread-member"""
        ...
    async def get_thread_member(
        self,
        *,
        channel_id: SnowflakeType,
        user_id: SnowflakeType,
        with_member: Optional[bool] = ...,
    ) -> ThreadMember:
        """https://discord.com/developers/docs/resources/channel#get-thread-member"""
        ...
    async def list_thread_members(
        self,
        *,
        channel_id: SnowflakeType,
        with_member: Optional[bool] = ...,
        after: Optional[SnowflakeType] = ...,
        limit: Optional[int] = ...,
    ) -> List[ThreadMember]:
        """https://discord.com/developers/docs/resources/channel#list-thread-members"""
        ...
    async def list_public_archived_threads(
        self,
        *,
        channel_id: SnowflakeType,
        before: Optional[datetime.datetime] = ...,
        limit: Optional[int] = ...,
    ) -> ArchivedThreadsResponse:
        """https://discord.com/developers/docs/resources/channel#list-public-archived-threads"""
        ...
    async def list_private_archived_threads(
        self,
        *,
        channel_id: SnowflakeType,
        before: Optional[datetime.datetime] = ...,
        limit: Optional[int] = ...,
    ) -> ArchivedThreadsResponse:
        """https://discord.com/developers/docs/resources/channel#list-private-archived-threads"""
        ...
    async def list_joined_private_archived_threads(
        self,
        *,
        channel_id: SnowflakeType,
        before: Optional[datetime.datetime] = ...,
        limit: Optional[int] = ...,
    ) -> ArchivedThreadsResponse:
        """https://discord.com/developers/docs/resources/channel#list-joined-private-archived-threads"""
        ...
    async def list_guild_emojis(self, *, guild_id: SnowflakeType) -> List[Emoji]:
        """https://discord.com/developers/docs/resources/emoji#list-guild-emojis"""
        ...
    async def get_guild_emoji(
        self, *, guild_id: SnowflakeType, emoji_id: SnowflakeType
    ) -> Emoji:
        """https://discord.com/developers/docs/resources/emoji#get-guild-emoji"""
        ...
    async def create_guild_emoji(
        self,
        *,
        guild_id: SnowflakeType,
        name: str = ...,
        image: str = ...,
        roles: List[SnowflakeType] = ...,
        reason: Optional[str] = ...,
    ) -> Emoji:
        """https://discord.com/developers/docs/resources/emoji#create-guild-emoji"""
        ...
    async def modify_guild_emoji(
        self,
        *,
        guild_id: SnowflakeType,
        emoji_id: str,
        name: str = ...,
        roles: Optional[List[SnowflakeType]] = ...,
        reason: Optional[str] = ...,
    ) -> Emoji:
        """https://discord.com/developers/docs/resources/emoji#modify-guild-emoji"""
        ...
    async def delete_guild_emoji(
        self, *, guild_id: SnowflakeType, emoji_id: str, reason: Optional[str] = ...
    ) -> None:
        """https://discord.com/developers/docs/resources/emoji#delete-guild-emoji"""
        ...
    async def create_guild(
        self,
        *,
        name: str = ...,
        region: Optional[str] = ...,
        icon: Optional[str] = ...,
        verification_level: Optional[VerificationLevel] = ...,
        default_message_notifications: Optional[DefaultMessageNotificationLevel] = ...,
        explicit_content_filter: Optional[ExplicitContentFilterLevel] = ...,
        roles: Optional[List["Role"]] = ...,
        channels: Optional[List[Channel]] = ...,
        afk_channel_id: Optional[Snowflake] = ...,
        afk_timeout: Optional[int] = ...,
        system_channel_id: Optional[Snowflake] = ...,
        system_channel_flags: Optional[SystemChannelFlags] = ...,
    ) -> Guild:
        """https://discord.com/developers/docs/resources/guild#create-guild"""
        ...
    async def get_guild(
        self, *, guild_id: SnowflakeType, with_counts: Optional[bool] = ...
    ) -> Guild:
        """https://discord.com/developers/docs/resources/guild#get-guild"""
        ...
    async def get_guild_preview(self, *, guild_id: SnowflakeType) -> GuildPreview:
        """https://discord.com/developers/docs/resources/guild#get-guild-preview"""
        ...
    async def modify_guild(
        self,
        *,
        guild_id: SnowflakeType,
        name: str = ...,
        region: Optional[str] = ...,
        verification_level: Optional[VerificationLevel] = ...,
        default_message_notifications: Optional[DefaultMessageNotificationLevel] = ...,
        explicit_content_filter: Optional[ExplicitContentFilterLevel] = ...,
        afk_channel_id: Optional[Snowflake] = ...,
        afk_timeout: Optional[int] = ...,
        icon: Optional[str] = ...,
        owner_id: Optional[Snowflake] = ...,
        splash: Optional[str] = ...,
        discovery_splash: Optional[str] = ...,
        banner: Optional[str] = ...,
        system_channel_id: Optional[Snowflake] = ...,
        system_channel_flags: Optional[SystemChannelFlags] = ...,
        rules_channel_id: Optional[Snowflake] = ...,
        public_updates_channel_id: Optional[Snowflake] = ...,
        preferred_locale: Optional[str] = ...,
        features: Optional[List[GuildFeature]] = ...,
        description: Optional[str] = ...,
        premium_progress_bar_enabled: Optional[bool] = ...,
        reason: Optional[str] = ...,
    ) -> Guild:
        """https://discord.com/developers/docs/resources/guild#modify-guild"""
        ...
    async def delete_guild(self, *, guild_id: SnowflakeType) -> None:
        """https://discord.com/developers/docs/resources/guild#delete-guild"""
        ...
    async def get_guild_channels(self, *, guild_id: SnowflakeType) -> List[Channel]:
        """https://discord.com/developers/docs/resources/guild#get-guild-channels"""
        ...
    async def create_guild_channel(
        self,
        *,
        guild_id: SnowflakeType,
        name: str = ...,
        type: Optional[ChannelType] = ...,
        topic: Optional[str] = ...,
        bitrate: Optional[int] = ...,
        user_limit: Optional[int] = ...,
        rate_limit_per_user: Optional[int] = ...,
        position: Optional[int] = ...,
        permission_overwrites: Optional[List["Overwrite"]] = ...,
        parent_id: Optional[Snowflake] = ...,
        nsfw: Optional[bool] = ...,
        rtc_region: Optional[str] = ...,
        video_quality_mode: Optional[VideoQualityMode] = ...,
        default_auto_archive_duration: Optional[int] = ...,
        default_reaction_emoji: Optional[DefaultReaction] = ...,
        available_tags: Optional[List[ForumTag]] = ...,
        default_sort_order: Optional[SortOrderTypes] = ...,
        reason: Optional[str] = ...,
    ) -> Channel:
        """https://discord.com/developers/docs/resources/guild#create-guild-channel"""
        ...
    async def modify_guild_channel_positions(
        self,
        *,
        guild_id: SnowflakeType,
        id: SnowflakeType = ...,
        position: Optional[int] = ...,
        lock_permissions: Optional[bool] = ...,
        parent_id: Optional[SnowflakeType] = ...,
    ) -> Guild:
        """https://discord.com/developers/docs/resources/guild#modify-guild-channel-positions"""
        ...
    async def list_active_guild_threads(
        self, *, guild_id: SnowflakeType
    ) -> ListActiveGuildThreadsResponse:
        """https://discord.com/developers/docs/resources/guild#list-active-guild-threads"""
        ...
    async def get_guild_member(
        self, *, guild_id: SnowflakeType, user_id: SnowflakeType
    ) -> GuildMember:
        """https://discord.com/developers/docs/resources/guild#get-guild-member"""
        ...
    async def list_guild_members(
        self,
        *,
        guild_id: SnowflakeType,
        limit: Optional[int] = ...,
        after: Optional[SnowflakeType] = ...,
    ) -> List[GuildMember]:
        """https://discord.com/developers/docs/resources/guild#list-guild-members"""
        ...
    async def search_guild_members(
        self,
        *,
        guild_id: SnowflakeType,
        query: Optional[str] = ...,
        limit: Optional[int] = ...,
    ) -> List[GuildMember]:
        """https://discord.com/developers/docs/resources/guild#search-guild-members"""
        ...
    async def add_guild_member(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        access_token: str = ...,
        nick: Optional[str] = ...,
        roles: Optional[List[SnowflakeType]] = ...,
        mute: Optional[bool] = ...,
        deaf: Optional[bool] = ...,
    ) -> GuildMember:
        """https://discord.com/developers/docs/resources/guild#add-guild-member"""
        ...
    async def modify_guild_member(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        nick: Optional[str] = ...,
        roles: Optional[List[SnowflakeType]] = ...,
        mute: Optional[bool] = ...,
        deaf: Optional[bool] = ...,
        channel_id: Optional[SnowflakeType] = ...,
        communication_disabled_until: Optional[datetime.datetime] = ...,
        flags: Optional[GuildMemberFlags] = ...,
        reason: Optional[str] = ...,
    ) -> GuildMember:
        """https://discord.com/developers/docs/resources/guild#modify-guild-member"""
        ...
    async def modify_current_member(
        self,
        *,
        guild_id: SnowflakeType,
        nick: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> GuildMember:
        """https://discord.com/developers/docs/resources/guild#modify-current-member"""
        ...
    async def modify_current_user_nick(
        self,
        *,
        guild_id: SnowflakeType,
        nick: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> GuildMember:
        """https://discord.com/developers/docs/resources/guild#modify-current-user-nick"""
        ...
    async def add_guild_member_role(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        role_id: SnowflakeType,
        reason: Optional[str] = ...,
    ):
        """https://discord.com/developers/docs/resources/guild#add-guild-member-role"""
        ...
    async def remove_guild_member_role(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        role_id: SnowflakeType,
        reason: Optional[str] = ...,
    ):
        """https://discord.com/developers/docs/resources/guild#remove-guild-member-role"""
        ...
    async def remove_guild_member(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        reason: Optional[str] = ...,
    ):
        """https://discord.com/developers/docs/resources/guild#remove-guild-member"""
        ...
    async def get_guild_bans(
        self,
        *,
        guild_id: SnowflakeType,
        limit: Optional[int] = ...,
        before: Optional[SnowflakeType] = ...,
        after: Optional[SnowflakeType] = ...,
    ) -> List[Ban]:
        """https://discord.com/developers/docs/resources/guild#get-guild-bans"""
        ...
    async def get_guild_ban(
        self, *, guild_id: SnowflakeType, user_id: SnowflakeType
    ) -> Ban:
        """https://discord.com/developers/docs/resources/guild#get-guild-ban"""
        ...
    async def create_guild_ban(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        delete_message_days: Optional[int] = ...,
        delete_message_seconds: Optional[int] = ...,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#create-guild-ban"""
        ...
    async def remove_guild_ban(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#remove-guild-ban"""
        ...
    async def get_guild_roles(self, *, guild_id: SnowflakeType) -> List[Role]:
        """https://discord.com/developers/docs/resources/guild#get-guild-roles"""
        ...
    async def create_guild_role(
        self,
        *,
        guild_id: SnowflakeType,
        name: Optional[str] = ...,
        permissions: Optional[str] = ...,
        color: Optional[int] = ...,
        hoist: Optional[bool] = ...,
        icon: Optional[str] = ...,
        unicode_emoji: Optional[str] = ...,
        mentionable: Optional[bool] = ...,
        reason: Optional[str] = ...,
    ) -> Role:
        """https://discord.com/developers/docs/resources/guild#create-guild-role"""
        ...
    async def modify_guild_role_positions(
        self,
        *,
        guild_id: SnowflakeType,
        id: SnowflakeType,
        position: Optional[int] = ...,
        reason: Optional[str] = ...,
    ) -> List[Role]:
        """https://discord.com/developers/docs/resources/guild#modify-guild-role-positions"""
        ...
    async def modify_guild_role(
        self,
        *,
        guild_id: SnowflakeType,
        role_id: SnowflakeType,
        name: Optional[str] = ...,
        permissions: Optional[str] = ...,
        color: Optional[int] = ...,
        hoist: Optional[bool] = ...,
        icon: Optional[str] = ...,
        unicode_emoji: Optional[str] = ...,
        mentionable: Optional[bool] = ...,
        reason: Optional[str] = ...,
    ) -> Role:
        """https://discord.com/developers/docs/resources/guild#modify-guild-role"""
        ...
    async def modify_guild_MFA_level(
        self, *, guild_id: SnowflakeType, level: int, reason: Optional[str] = ...
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#modify-guild-mfa-level"""
        ...
    async def delete_guild_role(
        self,
        *,
        guild_id: SnowflakeType,
        role_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#delete-guild-role"""
        ...
    async def get_guild_prune_count(
        self, *, guild_id: SnowflakeType, days: int, include_roles: List[SnowflakeType]
    ) -> Dict[Literal["pruned"], int]:
        """https://discord.com/developers/docs/resources/guild#get-guild-prune-count"""
        ...
    async def begin_guild_prune(
        self,
        *,
        guild_id: SnowflakeType,
        days: Optional[int] = ...,
        compute_prune_count: Optional[bool] = ...,
        include_roles: Optional[List[SnowflakeType]] = ...,
        reason: Optional[str] = ...,
    ) -> Dict[Literal["pruned"], int]:
        """https://discord.com/developers/docs/resources/guild#begin-guild-prune"""
        ...
    async def get_guild_voice_regions(
        self, *, guild_id: SnowflakeType
    ) -> List[VoiceRegion]:
        """https://discord.com/developers/docs/resources/guild#get-guild-voice-regions"""
        ...
    async def get_guild_invites(self, *, guild_id: SnowflakeType) -> List[Invite]:
        """https://discord.com/developers/docs/resources/guild#get-guild-invites"""
        ...
    async def get_guild_integrations(
        self, *, guild_id: SnowflakeType
    ) -> List[Integration]:
        """https://discord.com/developers/docs/resources/guild#get-guild-integrations"""
        ...
    async def delete_guild_integration(
        self,
        *,
        guild_id: SnowflakeType,
        integration_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#delete-guild-integration"""
        ...
    async def get_guild_widget_settings(
        self, *, guild_id: SnowflakeType
    ) -> GuildWidgetSettings:
        """https://discord.com/developers/docs/resources/guild#get-guild-widget-settings"""
        ...
    async def modify_guild_widget(
        self,
        *,
        guild_id: SnowflakeType,
        enabled: Optional[bool] = ...,
        channel_id: Optional[SnowflakeType] = ...,
        reason: Optional[str] = ...,
    ) -> GuildWidget:
        """https://discord.com/developers/docs/resources/guild#modify-guild-widget"""
        ...
    async def get_guild_widget(self, *, guild_id: SnowflakeType) -> GuildWidget:
        """https://discord.com/developers/docs/resources/guild#get-guild-widget"""
        ...
    async def get_guild_vanity_url(self, *, guild_id: SnowflakeType) -> Invite:
        """https://discord.com/developers/docs/resources/guild#get-guild-vanity-url"""
        ...
    async def get_guild_widget_image(
        self,
        *,
        guild_id: SnowflakeType,
        style: Optional[
            Literal["shield", "banner1", "banner2", "banner3", "banner4"]
        ] = ...,
    ) -> str:
        """https://discord.com/developers/docs/resources/guild#get-guild-widget-image"""
        ...
    async def get_guild_welcome_screen(
        self, *, guild_id: SnowflakeType
    ) -> WelcomeScreen:
        """https://discord.com/developers/docs/resources/guild#get-guild-welcome-screen"""
        ...
    async def modify_guild_welcome_screen(
        self,
        *,
        guild_id: SnowflakeType,
        enabled: Optional[bool] = ...,
        welcome_channels: Optional[List[WelcomeScreenChannel]] = ...,
        description: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> WelcomeScreen:
        """https://discord.com/developers/docs/resources/guild#modify-guild-welcome-screen"""
        ...
    async def get_guild_onboarding(self, *, guild_id: SnowflakeType) -> GuildOnboarding:
        """https://discord.com/developers/docs/resources/guild#get-guild-onboarding"""
        ...
    async def modify_current_user_voice_state(
        self,
        *,
        guild_id: SnowflakeType,
        channel_id: Optional[SnowflakeType] = ...,
        suppress: Optional[bool] = ...,
        request_to_speak_timestamp: Optional[datetime.datetime] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#modify-current-user-voice-state"""
        ...
    async def modify_user_voice_state(
        self,
        *,
        guild_id: SnowflakeType,
        user_id: SnowflakeType,
        channel_id: SnowflakeType,
        suppress: Optional[bool] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/guild#modify-user-voice-state"""
        ...
    async def list_scheduled_events_for_guild(
        self, *, guild_id: SnowflakeType, with_user_count: Optional[bool] = ...
    ) -> List[GuildScheduledEvent]:
        """https://discord.com/developers/docs/resources/guild-scheduled-event#list-scheduled-events-for-guild"""
        ...
    async def create_guild_schedule_event(
        self,
        *,
        guild_id: SnowflakeType,
        channel_id: Optional[Snowflake] = ...,
        entity_metadata: Optional[GuildScheduledEventEntityMetadata] = ...,
        name: str,
        privacy_level: GuildScheduledEventPrivacyLevel,
        scheduled_start_time: datetime.datetime,
        scheduled_end_time: Optional[datetime.datetime] = ...,
        description: Optional[str] = ...,
        entity_type: GuildScheduledEventEntityType,
        image: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> GuildScheduledEvent:
        """https://discord.com/developers/docs/resources/guild-scheduled-event#create-guild-scheduled-event"""
        ...
    async def get_guild_scheduled_event(
        self,
        *,
        guild_id: SnowflakeType,
        event_id: SnowflakeType,
        with_user_count: Optional[bool] = ...,
    ) -> GuildScheduledEvent:
        """https://discord.com/developers/docs/resources/guild-scheduled-event#get-guild-scheduled-event"""
        ...
    async def modify_guild_scheduled_event(
        self,
        *,
        guild_id: SnowflakeType,
        event_id: SnowflakeType,
        channel_id: Optional[Snowflake] = ...,
        entity_metadata: Optional[GuildScheduledEventEntityMetadata] = ...,
        name: Optional[str] = ...,
        privacy_level: Optional[GuildScheduledEventPrivacyLevel] = ...,
        scheduled_start_time: Optional[datetime.datetime] = ...,
        scheduled_end_time: Optional[datetime.datetime] = ...,
        description: Optional[str] = ...,
        entity_type: Optional[GuildScheduledEventEntityType] = ...,
        status: Optional[GuildScheduledEventStatus] = ...,
        image: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> GuildScheduledEvent:
        """https://discord.com/developers/docs/resources/guild-scheduled-event#modify-guild-scheduled-event"""
        ...
    async def delete_guild_scheduled_event(
        self, *, guild_id: SnowflakeType, event_id: SnowflakeType
    ) -> None:
        """https://discord.com/developers/docs/resources/guild-scheduled-event#delete-guild-scheduled-event"""
        ...
    async def get_guild_scheduled_event_users(
        self,
        *,
        guild_id: SnowflakeType,
        event_id: SnowflakeType,
        limit: Optional[int] = ...,
        with_member: Optional[bool] = ...,
        before: Optional[SnowflakeType] = ...,
        after: Optional[SnowflakeType] = ...,
    ) -> List[GuildScheduledEventUser]:
        """https://discord.com/developers/docs/resources/guild-scheduled-event#get-guild-scheduled-event-users"""
        ...
    async def get_guild_template(self, *, template_code: str) -> GuildTemplate:
        """https://discord.com/developers/docs/resources/guild-template#get-guild-template"""
        ...
    async def create_guild_from_guild_template(
        self, *, template_code: str, name: str, icon: Optional[str] = ...
    ) -> Guild:
        """https://discord.com/developers/docs/resources/guild-template#create-guild-from-template"""
        ...
    async def get_guild_templates(
        self, *, guild_id: SnowflakeType
    ) -> List[GuildTemplate]:
        """https://discord.com/developers/docs/resources/guild-template#get-guild-templates"""
        ...
    async def create_guild_template(
        self, *, guild_id: SnowflakeType, name: str, description: Optional[str] = ...
    ) -> GuildTemplate:
        """https://discord.com/developers/docs/resources/guild-template#create-guild-template"""
        ...
    async def sync_guild_template(
        self, *, guild_id: SnowflakeType, template_code: str
    ) -> GuildTemplate:
        """https://discord.com/developers/docs/resources/guild-template#sync-guild-template"""
        ...
    async def modify_guild_template(
        self,
        *,
        guild_id: SnowflakeType,
        template_code: str,
        name: Optional[str] = ...,
        description: Optional[str] = ...,
    ) -> GuildTemplate:
        """https://discord.com/developers/docs/resources/guild-template#modify-guild-template"""
        ...
    async def delete_guild_template(
        self, *, guild_id: SnowflakeType, template_code: str
    ) -> None:
        """https://discord.com/developers/docs/resources/guild-template#delete-guild-template"""
        ...
    async def get_invite(
        self,
        *,
        invite_code: str,
        with_counts: Optional[bool] = ...,
        with_expiration: Optional[bool] = ...,
        guild_scheduled_event_id: Optional[SnowflakeType] = ...,
    ) -> Invite:
        """https://discord.com/developers/docs/resources/invite#get-invite"""
        ...
    async def delete_invite(
        self, *, invite_code: str, reason: Optional[str] = ...
    ) -> Invite:
        """https://discord.com/developers/docs/resources/invite#delete-invite"""
        ...
    async def create_stage_instance(
        self,
        *,
        channel_id: SnowflakeType,
        topic: str,
        privacy_level: Optional[StagePrivacyLevel] = ...,
        send_start_notification: Optional[bool] = ...,
        reason: Optional[str] = ...,
    ) -> StageInstance:
        """https://discord.com/developers/docs/resources/stage-instance#create-stage-instance"""
        ...
    async def get_stage_instance(
        self, *, channel_id: SnowflakeType
    ) -> Optional[StageInstance]:
        """https://discord.com/developers/docs/resources/stage-instance#get-stage-instance"""
        ...
    async def modify_stage_instance(
        self,
        *,
        channel_id: SnowflakeType,
        topic: Optional[str] = ...,
        privacy_level: Optional[StagePrivacyLevel] = ...,
        reason: Optional[str] = ...,
    ) -> StageInstance:
        """https://discord.com/developers/docs/resources/stage-instance#modify-stage-instance"""
        ...
    async def delete_stage_instance(
        self, *, channel_id: SnowflakeType, reason: Optional[str] = ...
    ) -> None:
        """https://discord.com/developers/docs/resources/stage-instance#delete-stage-instance"""
        ...
    async def get_sticker(self, *, sticker_id: SnowflakeType) -> Sticker:
        """https://discord.com/developers/docs/resources/sticker#get-sticker"""
        ...
    async def list_nitro_sticker_packs(self) -> List[StickerPack]:
        """https://discord.com/developers/docs/resources/sticker#list-nitro-sticker-packs"""
        ...
    async def list_guild_stickers(self, *, guild_id: SnowflakeType) -> List[Sticker]:
        """https://discord.com/developers/docs/resources/sticker#list-guild-stickers"""
        ...
    async def get_guild_sticker(
        self, *, guild_id: SnowflakeType, sticker_id: SnowflakeType
    ) -> Sticker:
        """https://discord.com/developers/docs/resources/sticker#get-guild-sticker"""
        ...
    # async def create_guild_sticker(self,
    #                                *,
    #                                guild_id: SnowflakeType,
    #                                name: str,
    #                                description: str,
    #                                tags: str,
    #                                file: File,
    #                                reason: Optional[str] = ...) -> Sticker:
    #     """https://discord.com/developers/docs/resources/sticker#create-guild-sticker"""
    #     ...

    async def modify_guild_sticker(
        self,
        *,
        guild_id: SnowflakeType,
        sticker_id: SnowflakeType,
        name: Optional[str] = ...,
        description: Optional[str] = ...,
        tags: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> Sticker:
        """https://discord.com/developers/docs/resources/sticker#modify-guild-sticker"""
        ...
    async def delete_guild_sticker(
        self,
        *,
        guild_id: SnowflakeType,
        sticker_id: SnowflakeType,
        reason: Optional[str] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/sticker#delete-guild-sticker"""
        ...
    async def get_current_user(self) -> User:
        """https://discord.com/developers/docs/resources/user#get-current-user"""
        ...
    async def get_user(self, *, user_id: SnowflakeType) -> User:
        """https://discord.com/developers/docs/resources/user#get-user"""
        ...
    async def modify_current_user(
        self, *, username: Optional[str] = ..., avatar: Optional[str] = ...
    ) -> User:
        """https://discord.com/developers/docs/resources/user#modify-current-user"""
        ...
    async def get_current_user_guilds(
        self,
        *,
        before: Optional[SnowflakeType] = ...,
        after: Optional[SnowflakeType] = ...,
        limit: Optional[int] = ...,
    ) -> List[CurrentUserGuild]:
        """https://discord.com/developers/docs/resources/user#get-current-user-guilds"""
        ...
    async def get_current_user_guild_member(
        self, *, guild_id: SnowflakeType
    ) -> GuildMember:
        """https://discord.com/developers/docs/resources/user#get-current-user-guild-member"""
        ...
    async def leave_guild(self, *, guild_id: SnowflakeType) -> None:
        """https://discord.com/developers/docs/resources/user#leave-guild"""
        ...
    async def create_DM(self, *, recipient_id: SnowflakeType) -> Channel:
        """https://discord.com/developers/docs/resources/user#create-dm"""
        ...
    async def create_group_DM(
        self, *, access_tokens: List[str], nicks: Dict[SnowflakeType, str]
    ) -> Channel:
        """https://discord.com/developers/docs/resources/user#create-group-dm"""
        ...
    async def get_user_connections(self) -> List[Connection]:
        """https://discord.com/developers/docs/resources/user#get-user-connections"""
        ...
    async def get_user_application_role_connection(
        self, *, application_id: SnowflakeType
    ) -> ApplicationRoleConnection:
        """https://discord.com/developers/docs/resources/user#get-user-application-connections"""
        ...
    async def update_user_application_role_connection(
        self,
        *,
        application_id: SnowflakeType,
        platform_name: Optional[str] = ...,
        platform_username: Optional[str] = ...,
        metadata: Optional[ApplicationRoleConnectionMetadata] = ...,
    ) -> ApplicationRoleConnection:
        """https://discord.com/developers/docs/resources/user#modify-current-user"""
        ...
    async def list_voice_regions(self) -> List[VoiceRegion]:
        """https://discord.com/developers/docs/resources/voice#list-voice-regions"""
        ...
    async def create_webhook(
        self,
        *,
        channel_id: SnowflakeType,
        name: str,
        avatar: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> Webhook:
        """https://discord.com/developers/docs/resources/webhook#create-webhook"""
        ...
    async def get_channel_webhooks(self, *, channel_id: SnowflakeType) -> List[Webhook]:
        """https://discord.com/developers/docs/resources/webhook#get-channel-webhooks"""
        ...
    async def get_guild_webhooks(self, *, guild_id: SnowflakeType) -> List[Webhook]:
        """https://discord.com/developers/docs/resources/webhook#get-guild-webhooks"""
        ...
    async def get_webhook(self, *, webhook_id: SnowflakeType) -> Webhook:
        """https://discord.com/developers/docs/resources/webhook#get-webhook"""
        ...
    async def get_webhook_with_token(
        self, *, webhook_id: SnowflakeType, token: str
    ) -> Webhook:
        """https://discord.com/developers/docs/resources/webhook#get-webhook-with-token"""
        ...
    async def modify_webhook(
        self,
        *,
        webhook_id: SnowflakeType,
        name: Optional[str] = ...,
        avatar: Optional[str] = ...,
        channel_id: Optional[SnowflakeType] = ...,
        reason: Optional[str] = ...,
    ) -> Webhook:
        """https://discord.com/developers/docs/resources/webhook#modify-webhook"""
        ...
    async def modify_webhook_with_token(
        self,
        *,
        webhook_id: SnowflakeType,
        token: str,
        name: Optional[str] = ...,
        avatar: Optional[str] = ...,
    ) -> Webhook:
        """https://discord.com/developers/docs/resources/webhook#modify-webhook-with-token"""
        ...
    async def delete_webhook(
        self, *, webhook_id: SnowflakeType, reason: Optional[str] = ...
    ) -> None:
        """https://discord.com/developers/docs/resources/webhook#delete-webhook"""
        ...
    async def delete_webhook_with_token(
        self, *, webhook_id: SnowflakeType, token: str
    ) -> None:
        """https://discord.com/developers/docs/resources/webhook#delete-webhook-with-token"""
        ...
    # async def execute_webhook(self,
    #                           *,
    #
    #                           webhook_id: SnowflakeType,
    #                           token: str,
    #                           **data) -> None:
    #     """https://discord.com/developers/docs/resources/webhook#execute-webhook"""
    #     ...

    async def execute_slack_compatible_webhook(
        self,
        *,
        webhook_id: SnowflakeType,
        token: str,
        thread_id: Optional[SnowflakeType] = ...,
        wait: Optional[bool] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/webhook#execute-slackcompatible-webhook"""
        ...
    async def execute_github_compatible_webhook(
        self,
        *,
        webhook_id: SnowflakeType,
        token: str,
        thread_id: Optional[SnowflakeType] = ...,
        wait: Optional[bool] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/webhook#execute-githubcompatible-webhook"""
        ...
    async def get_webhook_message(
        self,
        *,
        webhook_id: SnowflakeType,
        token: str,
        message_id: SnowflakeType,
        thread_id: Optional[SnowflakeType] = ...,
    ) -> MessageGet:
        """https://discord.com/developers/docs/resources/webhook#get-webhook-message"""
        ...
    # async def edit_webhook_message(self) -> Message:
    #     """https://discord.com/developers/docs/resources/webhook#edit-webhook-message"""
    #     ...

    async def delete_webhook_message(
        self,
        *,
        webhook_id: SnowflakeType,
        token: str,
        message_id: SnowflakeType,
        thread_id: Optional[SnowflakeType] = ...,
    ) -> None:
        """https://discord.com/developers/docs/resources/webhook#delete-webhook-message"""
        ...
    async def get_gateway(self) -> Gateway:
        """https://discord.com/developers/docs/topics/gateway#get-gateway"""
        ...
    async def get_gateway_bot(self) -> GatewayBot:
        """https://discord.com/developers/docs/topics/gateway#get-gateway-bot"""
        ...
    async def get_current_bot_application_information(self) -> Application:
        """https://discord.com/developers/docs/resources/user#get-current-application-information"""
        ...
    async def get_current_authorization_information(self) -> AuthorizationResponse:
        """https://discord.com/developers/docs/resources/user#get-current-authorization-information"""
        ...



from .bard import load_config as load_bard_config
from .chatgpt_web import load_config as load_chat_gpt_web_config
from .claude_ai import load_config as load_claude_ai_config
from .newbing import load_config as load_newbing_config
from .poe import load_config as load_poe_config
from .slack_claude import load_config as load_slack_claude_config
from .spark_desk import load_config as load_spark_desk_config
from .tongyiqianwen import load_config as load_tongyiqianwen_config
from ..common.config import CONFIG_SOURCE
from ..common.load_config import load_command_config
from ..common.prefix_data import prefixes
from ..common.prompt_data import prompts

Generated_Source_Pic = False


def get_able_source():
    from .chatgpt_web import ABLE as CHATGPTWEB_ABLE
    from .newbing import ABLE as NEWBING_ABLE
    from .poe import ABLE as POE_ABLE
    from .slack_claude import ABLE as SLACKCLAUDE_ABLE
    from .claude_ai import ABLE as CLAUDE_AI_ABLE
    from .bard import ABLE as BARD_ABLE
    from .spark_desk import ABLE as SPARKDESK_ABLE
    from .tongyiqianwen import ABLE as TongYiQianWen_ABLE
    from .poe import SUBSCRIBE_ABLE as POE_SUBSCRIBE_ABLE
    able_dict = {
        "poe chatgpt": {
            "able": POE_ABLE,
            "des": "poe网站的chatgpt,支持预设最长约2000字,支持前缀但字数不宜过多",
        },
        "poe claude": {
            "able": POE_ABLE,
            "des": "poe网站的claude,支持预设最长约2000字,支持前缀但字数不宜过多",
        },
        "poe chatgpt4": {
            "able": POE_SUBSCRIBE_ABLE,
            "des": "poe网站的chatgpt4,仅限已定阅用户使用,仅限白名单用户对话使用,支持预设最长约2000字,支持前缀但字数不宜过多",
        },
        "poe claude-2-100k": {
            "able": POE_SUBSCRIBE_ABLE,
            "des": "poe网站的claude-2-100k,仅限已定阅用户使用,仅限白名单用户对话使用,支持预设最长约2000字,支持前缀但字数不宜过多",
        },
        "chatgpt web": {
            "able": CHATGPTWEB_ABLE,
            "des": "openai官网的chatgpt网页版,支持预设最长约5400字,支持前缀但字数不宜过多",
        },
        "claude ai": {
            "able": CLAUDE_AI_ABLE,
            "des": "claude官网的claude网页版,支持超极长的预设,有内容安全审查,支持前缀字数也很长",
        },
        "slack claude": {
            "able": SLACKCLAUDE_ABLE,
            "des": "slack网站的claude,支持非常长的预设,支持前缀但字数不宜过多",
        },
        "sydneybing": {
            "able": NEWBING_ABLE,
            "des": "微软的Newbing越狱版,支持长度约4000的预设,支持前缀但字数不宜过多",
        },
        "spark desk": {
            "able": SPARKDESK_ABLE,
            "des": "讯飞的讯飞星火语言模型,支持少数短预设和前缀",
        },
        "通义千问": {
            "able": TongYiQianWen_ABLE,
            "des": "阿里的通义千问,不支持预设",
        },
        "bing": {
            "able": NEWBING_ABLE,
            "des": "微软的Newbing,不支持预设",
        },
        "bard": {
            "able": BARD_ABLE,
            "des": "谷歌的Bard,不支持预设",
        },
    }

    source_dict = {}
    source_des_dict = {}
    i = 0
    for source, dict in able_dict.items():
        if dict["able"]:
            i += 1
            source_dict[str(i)] = source
            source_des_dict[source] = dict["des"]

    return source_dict, source_des_dict


def load_all_config():
    prompts.Generated = False
    prefixes.Generated = False
    load_command_config()
    load_bard_config()
    load_newbing_config()
    load_poe_config()
    load_slack_claude_config()
    load_claude_ai_config()
    load_chat_gpt_web_config()
    load_spark_desk_config()
    load_tongyiqianwen_config()


# 定义一个字典，把配置源和配置函数对应起来
CONFIG_DICT = {
    "总控配置": load_all_config,
    "Newbing配置": load_newbing_config,
    "Spark Desk配置": load_spark_desk_config,
    "Chat GPT web配置": load_chat_gpt_web_config,
    "Claude Slack配置": load_slack_claude_config,
    "Claude Ai配置": load_claude_ai_config,
    "Poe配置": load_poe_config,
    "Bard配置": load_bard_config,
    "通义千问配置": load_tongyiqianwen_config,
}


def load_config(source: CONFIG_SOURCE):
    global Generated_Source_Pic
    Generated_Source_Pic = False
    config_func = CONFIG_DICT.get(source)
    if config_func:
        config_func()
    else:
        raise Exception(f"load_config时error: 无效的配置源: {source}")


def get_source_pic():
    global Generated_Source_Pic
    Generated_Source_Pic = True



from typing_extensions import TypeAlias
from typing import Any, Dict, Union, Literal, TypeVar, final, Optional

T = TypeVar("T")


@final
class MissingType:
    """MissingType is a singleton class of MISSING.

    MISSING means that the field maybe not given in the data.

    MISSING Not equivalent to Python's Optional.

    see https://discord.com/developers/docs/reference#nullable-and-optional-resource-fields
    """

    _instance: Optional["MissingType"] = None

    def __new__(cls) -> "MissingType":
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __repr__(self) -> str:
        return "<MISSING>"

    def __str__(self) -> str:
        return self.__repr__()

    def __bool__(self) -> Literal[False]:
        return False

    def __copy__(self):
        return MISSING

    def __deepcopy__(self, memo: Dict[int, Any]):
        return MISSING

    @classmethod
    def __get_validators__(cls):
        yield cls._validate

    @classmethod
    def _validate(cls, value: Any):
        if value is not MISSING:
            raise ValueError(f"{value!r} is not MISSING type")
        return value


MISSING = MissingType()
"""MISSING means that the field maybe not given in the data.

see https://discord.com/developers/docs/reference#nullable-and-optional-resource-fields"""

Missing: TypeAlias = Union[MissingType, T]
"""Missing means that the field maybe not given in the data.

Missing[T] equal to Union[MISSING, T].

example: Missing[int] == Union[MISSING, int]"""

MissingOrNullable: TypeAlias = Union[MissingType, T, None]
"""MissingOrNullable means that the field maybe not given in the data or value is None.

MissingOrNullable[T] equal to Union[MISSING, T, None].

example: MissingOrNullable[int] == Union[MISSING, int, None]"""

__all__ = ["MISSING", "Missing", "MissingOrNullable"]



import sys
import json
import asyncio
import contextlib

# from pathlib import Path
from typing import Any, List, Tuple, Optional

from pydantic import parse_raw_as
from nonebot.typing import overrides
from nonebot.utils import escape_tag
from nonebot.exception import WebSocketClosed
from nonebot.drivers import URL, Driver, Request, WebSocket, ForwardDriver

from nonebot.adapters import Adapter as BaseAdapter

from .bot import Bot
from .config import Config, BotInfo
from .api.handle import API_HANDLERS
from .exception import ApiNotAvailable
from .api.model import User, GatewayBot
from .utils import log, decompress_data
from .event import Event, ReadyEvent, MessageEvent, event_classes
from .payload import (
    Hello,
    Resume,
    Payload,
    Dispatch,
    Identify,
    Heartbeat,
    Reconnect,
    PayloadType,
    HeartbeatAck,
    InvalidSession,
)

RECONNECT_INTERVAL = 3.0


class Adapter(BaseAdapter):
    @overrides(BaseAdapter)
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        self.discord_config: Config = Config(**self.config.dict())
        self.tasks: List[asyncio.Task] = []
        self.base_url: URL = URL(
            f"https://discord.com/api/v{self.discord_config.discord_api_version}"
        )
        self.setup()

    @classmethod
    @overrides(BaseAdapter)
    def get_name(cls) -> str:
        return "Discord"

    def setup(self) -> None:
        if not isinstance(self.driver, ForwardDriver):
            raise RuntimeError(
                f"Current driver {self.config.driver} doesn't support forward connections!"
                "Discord Adapter need a ForwardDriver to work."
            )
        self.driver.on_startup(self.startup)
        self.driver.on_shutdown(self.shutdown)

    async def startup(self) -> None:
        log("DEBUG", "Discord Adapter is starting up...")

        log("DEBUG", f"Discord api base url: <y>{escape_tag(str(self.base_url))}</y>")

        for bot_info in self.discord_config.discord_bots:
            self.tasks.append(asyncio.create_task(self.run_bot(bot_info)))

    async def shutdown(self) -> None:
        for task in self.tasks:
            if not task.done():
                task.cancel()

    async def run_bot(self, bot_info: BotInfo) -> None:
        try:
            gateway_info = await self._get_gateway_bot(bot_info)
            if not gateway_info.url:
                raise ValueError("Failed to get gateway url")
            ws_url = URL(gateway_info.url)
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Failed to get gateway info.</bg #f8bbd0></r>",
                e,
            )
            return
        remain = (
            gateway_info.session_start_limit
            and gateway_info.session_start_limit.remaining
        )
        if remain and remain <= 0:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Failed to establish connection to QQ Guild "
                "because of session start limit.</bg #f8bbd0></r>\n"
                f"{escape_tag(str(gateway_info))}",
            )
            return

        if bot_info.shard is not None:
            self.tasks.append(
                asyncio.create_task(self._forward_ws(bot_info, ws_url, bot_info.shard))
            )
            return

        shards = gateway_info.shards or 1
        for i in range(shards):
            self.tasks.append(
                asyncio.create_task(self._forward_ws(bot_info, ws_url, (i, shards)))
            )
            await asyncio.sleep(
                gateway_info.session_start_limit
                and gateway_info.session_start_limit.max_concurrency
                or 1
            )

    async def _get_gateway_bot(self, bot_info: BotInfo) -> GatewayBot:
        headers = {"Authorization": self.get_authorization(bot_info)}
        request = Request(
            headers=headers,
            method="GET",
            url=self.base_url / "gateway/bot",
            timeout=self.discord_config.discord_api_timeout,
            proxy=self.discord_config.discord_proxy,
        )
        resp = await self.request(request)
        if not resp.content:
            raise ValueError("Failed to get gateway info")
        return GatewayBot.parse_raw(resp.content)

    async def _get_bot_user(self, bot_info: BotInfo) -> User:
        headers = {"Authorization": self.get_authorization(bot_info)}
        request = Request(
            method="GET",
            url=self.base_url / "users/@me",
            headers=headers,
            timeout=self.discord_config.discord_api_timeout,
            proxy=self.discord_config.discord_proxy,
        )
        resp = await self.request(request)
        if not resp.content:
            raise ValueError("Failed to get bot user info")
        return User.parse_raw(resp.content)

    async def _forward_ws(
        self, bot_info: BotInfo, ws_url: URL, shard: Tuple[int, int]
    ) -> None:
        log("DEBUG", f"Forwarding WebSocket Connection to {escape_tag(str(ws_url))}...")
        headers = {"Authorization": self.get_authorization(bot_info)}
        params = {
            "v": self.discord_config.discord_api_version,
            "encoding": "json",
        }
        if self.discord_config.discord_compress:
            params["compress"] = "zlib-stream"
        request = Request(
            method="GET",
            url=ws_url,
            headers=headers,
            params=params,
            timeout=self.discord_config.discord_api_timeout,
            proxy=self.discord_config.discord_proxy,
        )
        heartbeat_task: Optional[asyncio.Task] = None
        bot: Optional[Bot] = None
        while True:
            try:
                if bot is None:
                    user = await self._get_bot_user(bot_info)
                    bot = Bot(self, str(user.id), bot_info)
                async with self.websocket(request) as ws:
                    ws: WebSocket
                    log(
                        "DEBUG",
                        f"WebSocket Connection to {escape_tag(str(ws_url))} established",
                    )
                    try:
                        # 接收hello事件
                        heartbeat_interval = await self._hello(ws)
                        if not heartbeat_interval:
                            await asyncio.sleep(RECONNECT_INTERVAL)
                            continue

                        if not heartbeat_task:
                            # 发送第一次心跳
                            log("DEBUG", "Waiting for first heartbeat to be send...")
                            # await asyncio.sleep(
                            #     random.random() * heartbeat_interval / 1000.0)  # https://discord.com/developers/docs/topics/gateway#heartbeat-interval
                            await asyncio.sleep(5)
                            await self._heartbeat(ws, bot)
                            await self._heartbeat_ack(ws)

                        # 开启心跳
                        heartbeat_task = asyncio.create_task(
                            self._heartbeat_task(ws, bot, heartbeat_interval)
                        )

                        # 进行identify和resume
                        result = await self._authenticate(bot, ws, shard)
                        if not result:
                            await asyncio.sleep(RECONNECT_INTERVAL)
                            continue

                        # 处理事件
                        await self._loop(bot, ws)
                    except WebSocketClosed as e:
                        log(
                            "ERROR",
                            "<r><bg #f8bbd0>WebSocket Closed</bg #f8bbd0></r>",
                            e,
                        )
                    except Exception as e:
                        log(
                            "ERROR",
                            "<r><bg #f8bbd0>Error while process data from websocket "
                            f"{escape_tag(str(ws_url))}. Trying to reconnect...</bg #f8bbd0></r>",
                            e,
                        )
                    finally:
                        if heartbeat_task:
                            heartbeat_task.cancel()
                            heartbeat_task = None
                        self.bot_disconnect(bot)

            except Exception as e:
                log(
                    "ERROR",
                    "<r><bg #f8bbd0>Error while setup websocket to "
                    f"{escape_tag(str(ws_url))}. Trying to reconnect...</bg #f8bbd0></r>",
                    e,
                )
                await asyncio.sleep(RECONNECT_INTERVAL)

    async def _hello(self, ws: WebSocket):
        """接收并处理服务器的 Hello 事件

        见 https://discord.com/developers/docs/topics/gateway#hello-event
        """
        try:
            payload = await self.receive_payload(ws)
            assert isinstance(
                payload, Hello
            ), f"Received unexpected payload: {payload!r}"
            log("DEBUG", f"Received hello: {payload}")
            return payload.data.heartbeat_interval
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Error while receiving server hello event</bg #f8bbd0></r>",
                e,
            )

    async def _heartbeat_ack(self, ws: WebSocket):
        """检查是否收到心跳ACK事件

        见 https://discord.com/developers/docs/topics/gateway#sending-heartbeats"""
        payload = await self.receive_payload(ws)
        if not isinstance(payload, HeartbeatAck):
            await ws.close(1003)  # 除了1000和1001都行

    @staticmethod
    async def _heartbeat(ws: WebSocket, bot: Bot):
        """心跳"""
        log("TRACE", f"Heartbeat {bot.sequence if bot.has_sequence else ''}")
        payload = Heartbeat.parse_obj(
            {"data": bot.sequence if bot.has_sequence else None}
        )
        with contextlib.suppress(Exception):
            await ws.send(json.dumps(payload.dict()))

    async def _heartbeat_task(self, ws: WebSocket, bot: Bot, heartbeat_interval: int):
        """心跳任务"""
        while True:
            await self._heartbeat(ws, bot)
            await asyncio.sleep(heartbeat_interval / 1000.0)

    async def _authenticate(self, bot: Bot, ws: WebSocket, shard: Tuple[int, int]):
        """鉴权连接"""
        if not bot.ready:
            payload = Identify.parse_obj(
                {
                    "data": {
                        "token": self.get_authorization(bot.bot_info),
                        "intents": bot.bot_info.intent.to_int(),
                        "shard": list(shard),
                        "compress": self.discord_config.discord_compress,
                        "properties": {
                            "os": sys.platform,
                            "browser": "NoneBot2",
                            "device": "NoneBot2",
                        },
                    },
                }
            )
        else:
            payload = Resume.parse_obj(
                {
                    "data": {
                        "token": self.get_authorization(bot.bot_info),
                        "session_id": bot.session_id,
                        "seq": bot.sequence,
                    }
                }
            )

        try:
            await ws.send(json.dumps(payload.dict(exclude_none=True)))
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Error while sending "
                + ("Identify" if isinstance(payload, Identify) else "Resume")
                + " event</bg #f8bbd0></r>",
                e,
            )
            return

        ready_event = None
        if not bot.ready:
            # https://discord.com/developers/docs/topics/gateway#ready-event
            # 鉴权成功之后，后台会下发一个 Ready Event
            payload = await self.receive_payload(ws)
            if isinstance(payload, HeartbeatAck):
                log(
                    "WARNING", "Received unexpected HeartbeatAck event when identifying"
                )
                payload = await self.receive_payload(ws)
            assert isinstance(
                payload, Dispatch
            ), f"Received unexpected payload: {payload!r}"
            bot.sequence = payload.sequence
            ready_event = self.payload_to_event(payload)
            assert isinstance(
                ready_event, ReadyEvent
            ), f"Received unexpected event: {ready_event!r}"
            ws.request.url = URL(ready_event.resume_gateway_url)
            bot.session_id = ready_event.session_id
            bot.self_info = ready_event.user

        # only connect for single shard
        if bot.self_id not in self.bots:
            self.bot_connect(bot)
            log(
                "INFO",
                f"<y>Bot {escape_tag(bot.self_id)}</y> connected",
            )

        if ready_event:
            asyncio.create_task(bot.handle_event(ready_event))

        return True

    async def _loop(self, bot: Bot, ws: WebSocket):
        """接收并处理事件"""
        while True:
            payload = await self.receive_payload(ws)
            log(
                "TRACE",
                f"Received payload: {escape_tag(repr(payload))}",
            )
            if isinstance(payload, Dispatch):
                bot.sequence = payload.sequence
                try:
                    event = self.payload_to_event(payload)
                except Exception as e:
                    # (Path() / f"{payload.type}-{payload.sequence}.json").write_text(
                    #     json.dumps(payload.dict(), indent=4, ensure_ascii=False),
                    #     encoding="utf-8",
                    # )
                    log(
                        "WARNING",
                        f"Failed to parse event {escape_tag(repr(payload))}",
                        e,
                    )
                else:
                    if not (
                        isinstance(event, MessageEvent)
                        and event.get_user_id() == bot.self_id
                        and not self.discord_config.discord_handle_self_message
                    ):
                        asyncio.create_task(bot.handle_event(event))
            elif isinstance(payload, Heartbeat):
                # 当接受到心跳payload时，需要立即发送一次心跳，见 https://discord.com/developers/docs/topics/gateway#heartbeat-requests
                await self._heartbeat(ws, bot)

            elif isinstance(payload, HeartbeatAck):
                log("TRACE", "Heartbeat ACK")
                continue
            elif isinstance(payload, Reconnect):
                log(
                    "WARNING",
                    "Received reconnect event from server. Try to reconnect...",
                )
                break
            elif isinstance(payload, InvalidSession):
                bot.clear()
                log(
                    "ERROR",
                    "Received invalid session event from server. Try to reconnect...",
                )
                break
            else:
                log(
                    "WARNING",
                    f"Unknown payload from server: {escape_tag(repr(payload))}",
                )

    @staticmethod
    def get_authorization(bot_info: BotInfo) -> str:
        return f"Bot {bot_info.token}"

    async def receive_payload(self, ws: WebSocket) -> Payload:
        data = await ws.receive()
        data = decompress_data(data, self.discord_config.discord_compress)
        return parse_raw_as(PayloadType, data)

    @classmethod
    def payload_to_event(cls, payload: Dispatch) -> Event:
        EventClass = event_classes.get(payload.type, None)
        if not EventClass:
            log(
                "WARNING",
                f"Unknown payload type: {payload.type}, detail: {repr(payload)}",
            )
            # (Path() / f"{payload.type}-{payload.sequence}.json").write_text(
            #     json.dumps(payload.dict(), indent=4, ensure_ascii=False),
            #     encoding="utf-8",
            # )
            event = Event.parse_obj(payload.data)
            event.__type__ = payload.type  # type: ignore
            return event
        return EventClass.parse_obj(payload.data)

    @overrides(BaseAdapter)
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        log("DEBUG", f"Calling API <y>{api}</y>")
        if (api_handler := API_HANDLERS.get(api)) is None:
            raise ApiNotAvailable
        return await api_handler(self, bot, **data)



import asyncio

from nonebot.log import logger

from .utils.EdgeGPT.EdgeGPT import ConversationStyle
from .utils.SydneyGPT.SydneyGPT import Chatbot
from ..common.mytypes import CommonUserInfo, BotData, BotInfo
from ..common.user_data import common_users


class SydneyBing_Bot:
    def __init__(
            self, common_userinfo: CommonUserInfo, bot_info: BotInfo, bot_data: BotData
    ):
        self.nickname = bot_info.nickname
        self.common_userinfo = common_userinfo
        self.botdata = bot_data
        self.chatbot: Chatbot = None
        self.chatstyle: ConversationStyle = ConversationStyle.creative
        self.lock = asyncio.Lock()

    def __hash__(self) -> int:
        return hash((self.nickname, self.common_userinfo.user_id))

    async def refresh(self):
        from .newbing import PROXY, COOKIES, WSS_LINK

        retry = 3
        detail_error = "未知错误"
        while retry > 0:
            try:
                if len(PROXY) > 0:
                    self.chatbot = await Chatbot.create(cookies=COOKIES, proxy=PROXY)
                else:
                    self.chatbot = await Chatbot.create(cookies=COOKIES)
                break
            except Exception as e:
                detail_error = str(e) if str(e) else str(e.__class__)
                logger.error(f"Sydneybing刷新时error:{detail_error}")
                retry -= 1
        if retry <= 0:
            error = f"Sydneybing刷新报错次数超过上限:{detail_error}"
            logger.error(error)
            raise Exception(error)
        if self.botdata.prompt and self.chatbot:
            if retry <= 0:
                error = f"Sydneybing刷新时报错次数超过上限{detail_error}"
                logger.error(error)
                raise Exception(error)
            if len(self.botdata.prompt) > 4000:
                raise Exception("Sydney的预设长度不得超过4000,请重新创建")
            else:
                retry = 2
                detail_error = "未知错误"
                while retry > 0:
                    try:
                        await self.chatbot.ask(
                            prompt=self.botdata.prompt,
                            conversation_style=ConversationStyle.creative,
                            simplify_response=True,
                            wss_link=WSS_LINK,
                            locale="en-us",
                        )
                        return
                    except Exception as e:
                        detail_error = str(e) if str(e) else str(e.__class__)
                        logger.error(f"Sydneybing在初始化预设时error:{detail_error}")
                        retry -= 1
            error = f"Sydneybing在初始化预设时出错次数超过上限{detail_error}"
            logger.error(error)
            raise Exception(error)
        else:
            return

    async def ask(self, question):
        from .newbing import WSS_LINK

        if question in ["creative", "创造", "balanced", "平衡", "precise", "精确"]:
            self.change_style(question)
            return f"成功切换为{question}模式"
        if not self.chatbot:
            await self.refresh()
        if question in ["1", "2", "3"] and self.botdata.last_suggests:
            question = self.botdata.last_suggests[int(question) - 1]

        if self.botdata.prefix:
            question = self.botdata.prefix + "\n" + question

        retry = 3
        detail_error = "未知错误"
        while retry >= 0:
            try:
                raw_json = (
                    await self.chatbot.ask(
                        prompt=question,
                        conversation_style=ConversationStyle.creative,
                        simplify_response=True,
                        wss_link=WSS_LINK,
                        locale="en-us",
                    ),
                )

                left, source_text, suggests = (
                    str(raw_json[-1]["messages_left"]),
                    "",
                    raw_json[-1]["suggestions"],
                )
                self.botdata.last_suggests = suggests

                if raw_json[-1]["sources_text"][:4] != raw_json[-1]["text"][:4]:
                    source_text = raw_json[-1]["sources_text"] + "\n"
                suggest_str = "\n".join(
                    [f"{i + 1}:{s}" for i, s in enumerate(suggests)]
                )
                answer = f"{raw_json[-1]['text'].replace('[^', '[').replace('^]', ']').replace('**', '')}\n\n{source_text}建议回复:\n{suggest_str}\n\n剩余{left}条连续对话"
                return answer
            except asyncio.TimeoutError:
                error = "Sydneybing在询问时超时无响应"
                logger.error(error)
                raise Exception(error)
            except Exception as e:
                detail_error = str(e)
                logger.warning(f"Sydneybing在询问时出错{detail_error}")
                if str(e) == "Max messages reached" or str(e) == "No message found":
                    await self.refresh()
                    while retry > 0:
                        try:
                            raw_json = await self.chatbot.ask(
                                prompt=question,
                                conversation_style=ConversationStyle.creative,
                                simplify_response=True,
                                wss_link=WSS_LINK,
                                locale="en-us",
                            )
                            left, source_text, suggests = (
                                str(raw_json[-1]["messages_left"]),
                                "",
                                raw_json[-1]["suggestions"],
                            )
                            self.botdata.last_suggests = suggests
                            if raw_json[-1]["sources_text"][:4] != raw_json[-1]["text"][:4]:
                                source_text = raw_json[-1]["sources_text"] + "\n"
                            suggest_str = "\n".join(
                                [f"{i + 1}:{s}" for i, s in enumerate(suggests)]
                            )
                            answer = f"{raw_json[-1]['text'].replace('[^', '[').replace('^]', ']').replace('**', '')}\n{source_text}建议回复:\n{suggest_str}\n剩余{left}条连续对话"
                            return answer
                        except Exception as e:
                            detail_error = str(e)
                            logger.error(f"SydenyBing在询问时出错:{detail_error}")
                            retry -= 1
                else:
                    retry -= 1
        error = f"Sydneybing询问时出错超过最大尝试数:{detail_error}"
        logger.error(error)
        raise Exception(error)

    def change_style(self, chat_style_str: str):
        if chat_style_str in ["creative", "创造"]:
            self.chatstyle = ConversationStyle.creative
        elif chat_style_str in ["balanced", "均衡"]:
            self.chatstyle = ConversationStyle.balanced
        elif chat_style_str in ["precise", "精确"]:
            self.chatstyle == ConversationStyle.precise
        common_users.save_userdata(common_userinfo=self.common_userinfo)
        return



from .SydneyGPT import Chatbot

try:
    from ..EdgeGPT import EdgeGPT as EdgeGPT_module
    from ..EdgeGPT.EdgeUtils import Query as BaseQuery
except ImportError:
    from .. import EdgeGPT as EdgeGPT_module
    from ..EdgeGPT.EdgeUtils import Query as BaseQuery

create_method = EdgeGPT_module.Chatbot.create


async def new_create(*args, **kwargs):
    monkey_create = EdgeGPT_module.Chatbot.create
    try:
        EdgeGPT_module.Chatbot.create = create_method
        gpt_bot_create = Chatbot.create(*args, **kwargs)
        return await gpt_bot_create
    finally:
        EdgeGPT_module.Chatbot.create = monkey_create


EdgeGPT_module.Chatbot.create = staticmethod(new_create)


class Query(BaseQuery):
    pass



from pathlib import Path

from nonebot import require
from nonebot.exception import MatcherException
from nonebot.matcher import Matcher
from nonebot.params import ArgStr
from nonebot.plugin import on_message
from nonebot.typing import T_State

require("nonebot_plugin_templates")
from nonebot_plugin_templates.templates_render import dict_render

from .utils import get_question_chatbot
from .utils import (
    if_super_user,
    set_common_userinfo,
    set_public_common_userinfo,
    delete_messages,
    send_message, send_img,
    if_close,
    MessageEvent,
    Bot,
)
from ..temp_bots import temp_bots
from ...chatbot.load_config import get_able_source
from ...common.mytypes import BotInfo, BotData
from ...common.prefix_data import prefixes
from ...common.prompt_data import prompts
from ...common.user_data import common_users
from ...utils.utils import is_valid_string

REFRESH_KEYWORDS = [
    "清除对话",
    "清空对话",
    "清除历史",
    "清空历史",
    "清除记录",
    "清空记录",
    "清空历史对话",
    "刷新对话",
]

chat = on_message(priority=1, block=False)


@chat.handle()
async def chat_(matcher: Matcher, event: MessageEvent, bot: Bot):
    from ...common.load_config import WAIT_MSG_ABLE

    question, chatbot, common_userinfo = await get_question_chatbot(event, bot, matcher)
    reply_msgs = []
    if chatbot.lock.locked():
        msg = "这个bot还有其他请求在处理,你的回复稍后就来"
        reply_msgs.append(await send_message(msg, matcher, bot, event, reply=True))

    if len(question) < 7:
        question = question.replace(" ", "").replace("\n", "")

    if question in REFRESH_KEYWORDS:
        async with chatbot.lock:
            if WAIT_MSG_ABLE == "True":
                reply_msgs.append(
                    await send_message("正在刷新，请稍等", matcher, bot, event, reply=True)
                )
            try:
                await chatbot.refresh()
                msg = "刷新对话成功"
            except Exception as e:
                msg = str(e)

    else:
        if len(question) >= 1:
            async with chatbot.lock:
                if WAIT_MSG_ABLE == "True":
                    reply_msgs.append(
                        await send_message("正在思考，请稍等", matcher, bot, event, reply=True)
                    )

                try:
                    msg = await chatbot.ask(question=question)
                except Exception as e:
                    msg = str(e)
        else:
            msg = "你没有输入任何问题,请重新询问并在命令后或回复中加上问题"

    try:
        await delete_messages(bot, event, reply_msgs)
    except Exception:
        pass
    reply = await send_message(msg, matcher, bot, event, plain=False, reply=True)
    temp_bots.set_bot_msgid(common_userinfo, chatbot, bot, event, reply)
    await matcher.finish()


new_bot = on_message(priority=1, block=False)
Source_Msg_Path = Path(__file__).parent / "SourceMsg.jpeg"
Source_Msg_Path.touch()


@new_bot.handle()
async def new_bot_(event: MessageEvent, matcher: Matcher, bot: Bot, state: T_State):
    from ...common.load_config import PRIVATE_COMMAND, PUBLIC_COMMAND
    from ...chatbot.load_config import Generated_Source_Pic, get_source_pic
    raw_message = event.get_plaintext()
    if not raw_message.startswith(
            (f"{PRIVATE_COMMAND}创建bot", f"{PUBLIC_COMMAND}创建bot")
    ):
        await matcher.finish()
    if raw_message.startswith(PUBLIC_COMMAND):
        await if_super_user(event, bot, matcher)
        state["pre_command"] = PUBLIC_COMMAND
        state["common_userinfo"] = set_public_common_userinfo(bot)
    else:
        state["pre_command"] = PRIVATE_COMMAND
        state["common_userinfo"] = set_common_userinfo(event, bot)

    state["able_source_dict"], source_des_dict = get_able_source()

    state["replys"] = []
    msg = "请输入要创建的bot的来源的序号\n输入'算了'或'取消'可以结束当前操作\n可选项如下图:"

    state["replys"].append(
        await send_message(
            msg,
            matcher,
            bot,
            event,
        )
    )
    if not Generated_Source_Pic:
        pic_bytes = await dict_render(source_des_dict, width=450, title="来源列表")
        get_source_pic()
        with open(Source_Msg_Path, "wb") as f:
            f.write(pic_bytes)
    else:
        with open(Source_Msg_Path, "rb") as f:
            pic_bytes = f.read()

    state["replys"].append(
        await send_img(
            pic_bytes,
            matcher,
            bot,
            event,
        )
    )


Prompt_Msg_Path = Path(__file__).parent / "PromptMsg.jpeg"
Prompt_Msg_Path.touch()


@new_bot.got("source_index")
async def new_bot__(
        matcher: Matcher,
        event: MessageEvent,
        state: T_State,
        bot: Bot,
        args: str = ArgStr("source_index"),
):
    await if_close(event, matcher, bot, state["replys"])

    state["source_index"] = str(args).replace("\n", "")
    if state["source_index"] not in [
        str(i) for i in range(1, len(state["able_source_dict"]) + 1)
    ]:
        await send_message("没有这个索引数字,请从头开始", matcher, bot, event)
        await delete_messages(bot, event, state["replys"])
        await matcher.finish()

    if not (
            state["able_source_dict"][state["source_index"]] == "bing"
            or state["able_source_dict"][state["source_index"]] == "bard"
            or state["able_source_dict"][state["source_index"]] == "通义千问"
    ):
        prompts_dict = prompts.show_list()
        msg = '请设置这个bot的预设\n预设是每次连续对话开始时发送初始化过去的内容,清除bot的对话历史后也会重新初始化\n\n如果不使用预设,请输入"无"或"无预设"\n如果使用本地预设,请输入预设前的数字索引\n如使用自己的预设直接发送即可\n\n输入"算了"或"取消"可以结束当前操作\n本地预设列表图片将在下面发送:'
        state["replys"].append(
            await send_message(
                msg, matcher, bot, event,
            )
        )

        if not prompts.Generated:
            pic_bytes = await dict_render(prompts_dict, title="预设列表", width=600)
            prompts.generate_pic()
            with open(Prompt_Msg_Path, "wb") as f:
                f.write(pic_bytes)
        else:
            with open(Prompt_Msg_Path, "rb") as f:
                pic_bytes = f.read()

        state["replys"].append(
            await send_img(
                pic_bytes, matcher, bot, event,
            )
        )
    else:
        state["prompt_nickname"] = "无预设"
        state["prefix_nickname"] = "无前缀"
        matcher.set_arg("prompt", "")
        matcher.set_arg("prefix", "")


Prefix_Msg_Path = Path(__file__).parent / "PrefixMsg.jpeg"
Prefix_Msg_Path.touch()


@new_bot.got("prompt")
async def new_bot____(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prompt"),
):
    await if_close(event, matcher, bot, state["replys"])
    prompt = str(args).replace("\n", "")
    prompt_nickname = "自定义预设"

    try:
        prompt_nickname = state["prompt_nickname"]
    except Exception:
        pass
    if prompt in ["无", "无预设"]:
        prompt = ""
        prompt_nickname = "无预设"
    elif prompt.isdigit():
        try:
            prompt_nickname, prompt = prompts.get_prompt(prompt)
        except Exception:
            await send_message("没有这个本地预设数字索引", matcher, bot, event)
            await matcher.finish()

    state["prompt_nickname"] = prompt_nickname
    state["prompt"] = prompt

    try:
        state["prefix_nickname"]
    except Exception:
        prefixes_dict = prefixes.show_list()
        msg = '请设置这个bot的前缀\n前缀是指每次对话时都在你的问题前自动添加的一些内容\n\n如果不使用前缀,请输入"无"或"无前缀"\n如果使用本地前缀,请发送前缀前的数字索引\n如果使用自己的前缀直接发送即可\n\n输入"算了"或"取消"可以结束当前操作\n本地前缀列表图片将在下面发送:'

        state["replys"].append(
            await send_message(
                msg, matcher, bot, event
            )
        )
        if not prefixes.Generated:
            pic_bytes = await dict_render(prefixes_dict, title="前缀列表", width=600)
            prefixes.generate_pic()
            with open(Prefix_Msg_Path, "wb") as f:
                f.write(pic_bytes)
        else:
            with open(Prefix_Msg_Path, "rb") as f:
                pic_bytes = f.read()
        state["replys"].append(
            await send_img(
                pic_bytes, matcher, bot, event
            )
        )


@new_bot.got("prefix")
async def new_bot______(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("prefix"),
):
    await if_close(event, matcher, bot, state["replys"])
    prefix_nickname = "自定义前缀"

    try:
        prefix_nickname = state["prefix_nickname"]
    except Exception:
        pass

    prefix = str(args)
    if prefix in ["无", "无前缀"]:
        prefix = ""
        prefix_nickname = "无前缀"
    elif prefix.isdigit():
        try:
            prefix_nickname, prefix = prefixes.get_prefix(prefix)
        except Exception:
            await send_message("没有这个本地前缀数字索引", matcher, bot, event)
            await matcher.finish()
    state["prefix"] = prefix
    state["prefix_nickname"] = prefix_nickname

    state["replys"].append(
        await send_message(
            "请为这个新bot设置一个独一无二的昵称\n只允许使用中文,英文,数字组成\n输入'算了'或'取消'可以结束当前操作",
            matcher,
            bot,
            event,
        )
    )


@new_bot.got("bot_nickname")
async def new_bot___(
        matcher: Matcher,
        event: MessageEvent,
        state: T_State,
        bot: Bot,
        args: str = ArgStr("bot_nickname"),
):
    await if_close(event, matcher, bot, state["replys"])

    bot_nickname = str(args).replace("\n", "").replace("\r", "").replace(" ", "")

    if not is_valid_string(bot_nickname):
        msg = "bot的名称不能包含特殊字符,只允许中文英文和数字,请重新开始"
        await send_message(msg, matcher, bot, event)
        await delete_messages(bot, event, state["replys"])
        await matcher.finish()
    else:
        prefix = state["prefix"]
        prefix_nickname = state["prefix_nickname"]
        common_userinfo = state["common_userinfo"]

        prompt_nickname = state["prompt_nickname"]
        prompt = state["prompt"]

        botinfo = BotInfo(nickname=bot_nickname, owner=common_userinfo)
        try:
            temp_bots.add_new_bot(
                common_userinfo=common_userinfo,
                botinfo=botinfo,
                botdata=BotData(
                    nickname=bot_nickname,
                    prompt_nickname=prompt_nickname,
                    prompt=prompt,
                    prefix_nickname=prefix_nickname,
                    prefix=prefix,
                    source=state["able_source_dict"][state["source_index"]],
                ),
            )
            pre_command = state["pre_command"]
            msg = f"成功添加了bot,使用命令{pre_command}{bot_nickname}加上你要询问的内容即可使用该bot"
            await send_message(msg, matcher, bot, event)
            await matcher.finish()
        except MatcherException:
            await delete_messages(bot, event, state["replys"])
            raise
        except Exception as e:
            await send_message(str(e), matcher, bot, event)
            await matcher.finish()

delete_bot = on_message(priority=1, block=False)


@delete_bot.handle()
async def delete_bot_(matcher: Matcher, event: MessageEvent, bot: Bot, state: T_State):
    from ...common.load_config import PRIVATE_COMMAND, PUBLIC_COMMAND

    if not event.get_plaintext().startswith(
            (f"{PRIVATE_COMMAND}删除bot", f"{PUBLIC_COMMAND}删除bot")
    ):
        await matcher.finish()
    if event.get_plaintext().startswith(PRIVATE_COMMAND):
        state["common_userinfo"] = set_common_userinfo(event, bot)
        plain_message = (
            event.get_plaintext()
            .replace(f"{PRIVATE_COMMAND}删除bot", "")
            .replace(" ", "")
        )
    else:
        await if_super_user(event, bot, matcher)
        state["common_userinfo"] = set_public_common_userinfo(bot)
        plain_message = (
            event.get_plaintext().replace(f"{PUBLIC_COMMAND}删除bot", "").replace(" ", "")
        )
    state["replys"] = []
    if not plain_message:
        msg = "请输入bot的昵称,区分大小写\n输入'取消'或'算了'可以结束当前操作"
        state["replys"].append(await send_message(msg, matcher, bot, event))

    else:
        matcher.set_arg("bot", plain_message)


@delete_bot.got("bot")
async def delete_bot__(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("bot"),
):
    await if_close(event, matcher, bot, state["replys"])
    bot_name = str(args).replace("\n", "")
    common_userinfo = state["common_userinfo"]
    try:
        common_users.delete_bot(
            common_userinfo=common_userinfo,
            botinfo=BotInfo(nickname=bot_name, owner=common_userinfo),
        )
        await send_message("成功删除了该bot", matcher, bot, event)
        await matcher.finish()
    except MatcherException:
        await delete_messages(bot, event, state["replys"])
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


rename_bot = on_message(priority=1, block=False)


@rename_bot.handle()
async def rename_bot_(matcher: Matcher, bot: Bot, event: MessageEvent, state: T_State):
    from ...common.load_config import PRIVATE_COMMAND, PUBLIC_COMMAND

    if not event.get_plaintext().startswith(
            (f"{PUBLIC_COMMAND}改名bot", f"{PRIVATE_COMMAND}改名bot")
    ):
        await matcher.finish()

    if event.get_plaintext().startswith(PRIVATE_COMMAND):
        state["common_userinfo"] = set_common_userinfo(event=event, bot=bot)
    else:
        await if_super_user(event, bot, matcher)
        state["common_userinfo"] = set_public_common_userinfo(bot)
    msg = "请输入要更改的bot的名称\n输入'取消'或'算了'可以结束当前操作"
    state["replys"] = []
    state["replys"].append(await send_message(msg, matcher, bot, event))


@rename_bot.got("bot_name")
async def rename_bot__(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("bot_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    state["bot_name"] = str(args)
    state["replys"].append(
        await send_message(
            "请输入bot要改为的名字\n注意bot的名字只能由中文,英文,数字组成\n输入'取消'或'算了'可以结束当前操作",
            matcher,
            bot,
            event,
        )
    )


@rename_bot.got("new_bot_name")
async def rename_bot___(
        matcher: Matcher,
        state: T_State,
        bot: Bot,
        event: MessageEvent,
        args: str = ArgStr("new_bot_name"),
):
    await if_close(event, matcher, bot, state["replys"])
    new_botname = str(args).replace("\n", "").replace("\r", "").replace(" ", "")
    if not is_valid_string(new_botname):
        await send_message("bot名称不能包含特殊字符,请重新开始", matcher, bot, event)
        await delete_messages(bot, event, state["replys"])
        await matcher.finish()
    try:
        common_users.rename_bot(
            state["common_userinfo"], state["bot_name"], new_botname
        )
        await send_message("成功更改了对应的bot的名称", matcher, bot, event)
        await matcher.finish()

    except MatcherException:
        await delete_messages(bot, event, state["replys"])
        raise
    except Exception as e:
        await send_message(str(e), matcher, bot, event)
        await matcher.finish()


all_bots = on_message(priority=1, block=False)


@all_bots.handle()
async def all_bots_(matcher: Matcher, bot: Bot, event: MessageEvent):
    from ...common.load_config import PRIVATE_COMMAND, PUBLIC_COMMAND

    if not event.get_plaintext().startswith(
            (f"{PRIVATE_COMMAND}所有bot", f"{PUBLIC_COMMAND}所有bot")
    ):
        await matcher.finish()

    if event.get_plaintext().startswith(PRIVATE_COMMAND):
        pre_command = PRIVATE_COMMAND
        common_userinfo = set_common_userinfo(event=event, bot=bot)
    else:
        pre_command = PUBLIC_COMMAND
        common_userinfo = set_public_common_userinfo(bot)

    await send_message(
        common_users.show_all_bots(common_userinfo, pre_command),
        matcher,
        bot,
        event,
        plain=False,
        forcepic=True,
        width=500,
    )
    await matcher.finish()



import re
import uuid

import aiohttp


def generate_uuid():
    return str(uuid.uuid4()).replace("-", "")[0:10]


async def get_url(text):
    """将 Markdown 文本保存到 Mozilla Pastebin,并获得 URL"""
    async with aiohttp.ClientSession() as session:
        payload = {
            "expires": "86400",
            "format": "url",
            "lexer": "_markdown",
            "content": text,
        }
        retries = 3
        while retries > 0:
            try:
                async with session.post(
                        "https://pastebin.mozilla.org/api/", data=payload
                ) as resp:
                    resp.raise_for_status()
                    url = await resp.text()
                    url = url[0:-1]
                    return url
            except Exception as e:
                retries -= 1
                if retries == 0:
                    url = f"上传失败：{str(e)}"
                    return url


def is_valid_string(s):
    pattern = re.compile(r"^[\u4e00-\u9fa5a-zA-Z0-9]+$")
    return bool(pattern.match(s))



from typing import TYPE_CHECKING, Any

from nonebot.drivers import Request
from nonebot.utils import escape_tag

from ..utils import log, json_loads, decompress_data
from ..exception import (
    ActionFailed,
    NetworkError,
    ApiNotAvailable,
    RateLimitException,
    UnauthorizedException,
    DiscordAdapterException,
)

if TYPE_CHECKING:
    from ..bot import Bot
    from ..adapter import Adapter


async def _request(adapter: "Adapter", bot: "Bot", request: Request) -> Any:
    try:
        request.timeout = adapter.discord_config.discord_api_timeout
        request.proxy = adapter.discord_config.discord_proxy
        data = await adapter.request(request)
        log(
            "TRACE",
            f"API code: {data.status_code} response: {escape_tag(str(data.content))}",
        )
        if data.status_code in (200, 201, 204):
            return data.content and json_loads(
                decompress_data(data.content, adapter.discord_config.discord_compress)
            )
        elif data.status_code in (401, 403):
            raise UnauthorizedException(data)
        elif data.status_code in (404, 405):
            raise ApiNotAvailable
        elif data.status_code == 429:
            raise RateLimitException(data)
        else:
            raise ActionFailed(data)
    except DiscordAdapterException:
        raise
    except Exception as e:
        raise NetworkError("API request failed") from e
